// THIS EXPECTS ALL CSS AND HTML ARE EXISTING
class Pixelate {
    constructor(src, callback, options) {
        this.src = src;
        this.image = false;
        this.canvasName = null;
        this.canvas = false;
        this.ctx = null;
        this.w = 24;
        this.src_w = 0;
        this.src_h = 0;
        this.canvasName = 'canvas-render'; // Canvas element by id
        this.callback = false;

        this.canvas_end_w = 0;
        this.canvas_end_h = 0;

        this.options = {
            smooth: true,
            size: 1,
            canvas_width: 640,
            canvas_height: 640,
            filter: false,
            colors: false,
            orientation: false,
            platform: 'android',
        }

        if(options)
        {
            this.options = $.extend({}, this.options, options);
        }

        // Render Image
        this.process(callback); 
    }

    run(options, callback) {
        if(options)
        {
            this.options = $.extend({}, this.options, options);
        }

        this.callback = callback;
        
        this.checkCanvas(); // Checks to see if canvas exist or not

        this.set(); // Creates pixel sizes
        this.render(callback); // Render
    }

    checkCanvas() {
        if(!this.canvas)
        {
            this.canvas = document.getElementById(this.canvasName);
            this.createCtx();
        }
    }

    createCtx() {
        this.ctx = this.canvas.getContext("2d");
    }

    smooth(status) {
        this.ctx.imageSmoothingEnabled = status;
        this.ctx.mozImageSmoothingEnabled = status;
        this.ctx.webkitImageSmoothingEnabled = status;
        this.ctx.msImageSmoothingEnabled = status;
    }

    process(callback) {
        let _this = this;
        let i = new Image;

        i.onload = function() {
            _this.src_w = this.width; // Defaults
            _this.src_h = this.height; // Defaults
            _this.w = this.width; // Defaults
            _this.h = this.height; // Defaults

            _this.setSizes(); 
            _this.resize(i, callback);
        }

        i.src = this.src;
    }

    resize(image, callback) {
        let _this = this;
        this.checkCanvas(); // Checks to see if canvas exist or not
        // Draw on image and get new src
        this.canvas.width = this.canvas_end_w;
        this.canvas.height = this.canvas_end_h;

        if(this.options.orientation && this.options.platform == 'ios')
        {
            if(this.options.orientation > 4 && this.options.orientation < 9)
            {
                this.canvas.height = this.canvas_end_w;
                this.canvas.width = this.canvas_end_h;
            }

            switch(this.options.orientation) 
            {
                case 2: this.ctx.transform(-1, 0, 0, 1, this.canvas.height, 0); break;
                case 3: this.ctx.transform(-1, 0, 0, -1, this.canvas.width, this.canvas.height); break;
                case 4: this.ctx.transform(1, 0, 0, -1, 0, this.canvas.width); break;
                case 5: this.ctx.transform(0, 1, 1, 0, 0, 0); break;
                case 6: this.ctx.transform(0, 1, -1, 0, this.canvas.width, 0); break;
                case 7: this.ctx.transform(0, -1, -1, 0, this.canvas.width, this.canvas.height); break;
                case 8: this.ctx.transform(0, -1, 1, 0, 0, this.canvas.height); break;
                default: this.ctx.transform(1, 0, 0, 1, 0, 0);
            }
        }

        // For orientation
        if(this.options.orientation && this.options.platform == 'ios' && this.options.orientation > 4 && this.options.orientation < 9)
        {
            this.ctx.drawImage(image, 0, 0, this.canvas.height, this.canvas.width);
        }
        else
        {
            this.ctx.drawImage(image, 0, 0, this.canvas.width, this.canvas.height);   
        }

        let i = new Image;

        i.onload = function() {
            _this.image = this;
            _this.src_w = this.width; // Defaults
            _this.src_h = this.height; // Defaults
            _this.w = this.width; // Defaults
            _this.h = this.height; // Defaults

            _this.setSizes(); 

            i = null; // Clear memory
            image = null; // Clear memory

            if(_this.options.filter)
            {
                return _this.filter(callback);
            }

            if(callback)
            {
                callback(this);
            }
        }

        i.src = this.canvas.toDataURL();
    }

    set() {
        return this.w = this.image.width * this.options.size;
    }

    render(callback) {
        this.setCanvasSize();

        // Draw Image on canvas
        this.ctx.drawImage(this.image, 0, 0, this.canvas.width, this.canvas.height);

        // Now redraw image large
        this.enlarge(callback);
    }

    // Canvas enlarging
    enlarge(callback) {
        let _this = this;
        let c = this.ctx.getImageData(0, 0, this.canvas.width, this.canvas.height);

        // Apply changes
        this.ctx.putImageData(c, 0, 0);

        // Now get base
        let b = this.canvas.toDataURL();
        let i = new Image;

        _this.smooth(true);

        i.onload = function() {
            let w = _this.src_w / _this.w;

            _this.canvas.width = _this.canvas_end_w;
            _this.canvas.height = _this.canvas_end_h;

            if(_this.options.smooth)
            {
                _this.smooth(false); // Set false
            } 

            _this.ctx.drawImage(this, 0, 0, _this.canvas_end_w, _this.canvas_end_h);

            i = null; // Clear memory

            return callback(_this.canvas.toDataURL());
        }

        i.src = b;
    }

    // Get sizes for canvas to see if canvas is too large
    setSizes() {
        var w = this.src_w;
        var h = this.src_h;

        if(this.src_w > this.options.canvas_width)
        {
            this.src_w = this.options.canvas_width;
            this.src_h = h / (w / this.src_w);
        }
        
        if(this.src_h > this.options.canvas_width)
        {
            this.src_h = this.options.canvas_height;
            this.src_w = w / (h / this.src_h);
        }

        this.canvas_end_w = this.src_w;
        this.canvas_end_h = (h / (w / this.src_w));
    }

    setCanvasSize() {
        this.canvas.width = this.w;
        this.canvas.height = this.getHeight(this.image.width, this.image.height);
    }

    getHeight(width, height) {
        return this.h = height / (width / this.w);
    }

    hexToRgb(hex) {
        let result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);

        return result ? {
            r: parseInt(result[1], 16),
            g: parseInt(result[2], 16),
            b: parseInt(result[3], 16)
        } : null;
    }

    // applied right away
    filter(callback) {
        let _this = this;
        let image = this.ctx.getImageData(0, 0, this.canvas_end_w, this.canvas_end_h);
        let data = image.data;
        let len = data.length;

        // Color Filter
        let currentPreset = this.options.colors;

        let presetLen = currentPreset.length;
        let currentColorArray = [];

        for(let x = 0; x < (currentPreset.length); x++)
        {
            let rgb = this.hexToRgb(currentPreset[x]);
            currentColorArray.push(rgb);
        }

        if(currentColorArray.length > 400)
        {
            return;
        }

        for(let p = 0; p < len; p += 4)
        {
            let lowestColor = false;
            let lowestIndex = 0;

            let r = data[p];
            let g = data[p + 1];
            let b = data[p + 2];
            let a = data[p + 3];

            if(data[p + 3] == 0)
            {   
                continue;   
            }

            // Loop each colaor in pallet
            for(let x = 0; x < presetLen; x++)
            {
                let index = x;
                let value = currentPreset[x];

                let rgb = currentColorArray[x];
                let rd = Math.abs(rgb.r - r) + Math.abs(rgb.g - g) + Math.abs(rgb.b - b);

                if(rd <= lowestColor || lowestColor === false)
                {
                    lowestIndex = rgb;
                    lowestColor = rd;
                }

                if(x == (currentPreset.length - 1))
                {
                    data[p] = lowestIndex.r;
                    data[p + 1] = lowestIndex.g;
                    data[p + 2] = lowestIndex.b;
                }
            }
        }

        this.ctx.putImageData(image, 0, 0);

        let i = new Image;

        // Setting image filter data
        i.onload = function() {
            _this.image = this;

            i = null; // Clear memory
            data = null; // Clear memory
            image = null; // Clearing memory

            callback(this);
        }

        i.src = this.canvas.toDataURL();

        data = null; // Clear memory
        image = null; // Clearing memory
    }
}

export { Pixelate as default }